package at.gv.egiz.moazs.repository;


import at.gv.egiz.moazs.scheme.MsgResponse;
import at.gv.zustellung.app2mzs.xsd.DeliveryRequestType;
import com.google.common.cache.Cache;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Profile;
import org.springframework.stereotype.Repository;

import java.util.Optional;
import java.util.concurrent.TimeUnit;

import static com.google.common.cache.CacheBuilder.newBuilder;
import static java.util.Optional.ofNullable;

@Repository
@Profile("!cluster")
public class InMemoryDeliveryRepository implements DeliveryRepository {

    private static final Logger log = LoggerFactory.getLogger(InMemoryDeliveryRepository.class);

    private final Cache<String, DeliveryRequestType> requestRepository;
    private final Cache<String, MsgResponse> responseRepository;
    private final Cache<String, byte[]> binaryResponseRepository;

    public InMemoryDeliveryRepository(int expiresAfterWrite) {
        var builder = newBuilder().expireAfterWrite(expiresAfterWrite, TimeUnit.MINUTES);
        this.requestRepository = builder.build();
        this.responseRepository = builder.build();
        this.binaryResponseRepository = builder.build();
    }

    //TODO: Handle all "key exists" cases!
    @Override
    public void store(DeliveryRequestType request) {
        var key = request.getMetaData().getAppDeliveryID();
        log.info("Store DeliveryRequest by key={}.", key);
        requestRepository.put(key, request);
    }

    @Override
    public Optional<DeliveryRequestType> retrieveDeliveryRequest(String appDeliveryID) {
        log.info("Retrieve DeliveryRequest by key={}.", appDeliveryID);
        return ofNullable(requestRepository.getIfPresent(appDeliveryID));
    }

    @Override
    public void store(MsgResponse response) {
        String key = response.getResponseID();
        log.info("Store MsgResponse by key={}.", key);
        responseRepository.put(key, response);
    }

    @Override
    public Optional<MsgResponse> retrieveResponse(String responseID) {
        log.info("Retrieve MsgResponse by key={}.", responseID);
        return ofNullable(responseRepository.getIfPresent(responseID));
    }

    @Override
    public void store(String responseID, byte[] data) {
        log.info("Store Binary MsgResponse by key={}.", responseID);
        binaryResponseRepository.put(responseID, data);
    }

    @Override
    public Optional<byte[]> retrieveBinaryResponse(String responseID) {
        log.info("Retrieve Binary MsgResponse by key={}.", responseID);
        return ofNullable(binaryResponseRepository.getIfPresent(responseID));
    }

}
